"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.genData = exports.genWhere = exports.getFields = exports.ErrorUnknownOperator = exports.ErrorWhereInEmptyArray = exports.ErrorSetNull = exports.ErrorWhereNull = void 0;
const _ = require("lodash");
const types_1 = require("./types");
exports.ErrorWhereNull = "where cannot be null or empty";
exports.ErrorSetNull = "set data cannot be null or empty";
exports.ErrorWhereInEmptyArray = "where condition [in] value must be array (len>0)";
exports.ErrorUnknownOperator = "unknown operator, only " + JSON.stringify(types_1.AvailableOperator) + " is available";
const getFiled = (field) => "`" + field + "`";
const getFields = (fields) => (fields != null && fields !== "*") ? fields.split(",").map(e => getFiled(e)).join(", ") : "*";
exports.getFields = getFields;
const withOperator = (field, operator) => {
    const sqlArr = [];
    const orSqlArr = [];
    let args = [];
    let orArgs = [];
    for (const opr in operator) {
        if (!types_1.AvailableOperator.includes(opr)) {
            throw new Error(exports.ErrorUnknownOperator);
        }
        if (opr === types_1.OR) {
            const tmpOpr = withOperator(field, operator[opr]);
            if (tmpOpr.ok === false) {
                continue;
            }
            orSqlArr.push(tmpOpr.sql);
            orArgs = orArgs.concat(tmpOpr.args);
            continue;
        }
        let oprValue = operator[opr];
        if (opr === "IN") {
            if (!Array.isArray(oprValue) || oprValue.length === 0) {
                throw new Error(exports.ErrorWhereInEmptyArray);
            }
            oprValue = [oprValue];
        }
        sqlArr.push(`${field} ${opr} ?`);
        args.push(oprValue);
    }
    let sql = sqlArr.join(" AND ");
    if (orSqlArr.length > 0) {
        sql = "((" + sql + ") OR (" + orSqlArr.join(" AND ") + "))";
        args = args.concat(orArgs);
    }
    sql = sql.replace(/ AND\s+OR /g, " OR ")
        .replace(/^\s+OR /g, "");
    return { sql, args, ok: sqlArr.length > 0 || orSqlArr.length > 0 };
};
const genWhere = (where) => {
    if (where == null) {
        return { sql: "", args: [], ok: false };
    }
    where = _.cloneDeep(where);
    const sqlArr = [];
    let args = [];
    const orSqlArr = [];
    let orArgs = [];
    for (const key in where) {
        if (key === types_1.OR) {
            const tmpWhere = exports.genWhere(where[key]);
            if (tmpWhere.ok === false) {
                continue;
            }
            orSqlArr.push(tmpWhere.sql);
            orArgs = orArgs.concat(tmpWhere.args);
            continue;
        }
        const value = where[key];
        const field = getFiled(key);
        if (value == null) {
            sqlArr.push(`${field} IS NULL`);
            continue;
        }
        if (typeof value === "string" || typeof value === "number") {
            sqlArr.push(`${field} = ?`);
            args.push(value);
            continue;
        }
        const operator = value;
        const operatorSql = withOperator(field, operator);
        if (operatorSql.ok === true) {
            sqlArr.push(operatorSql.sql);
            args = args.concat(operatorSql.args);
        }
    }
    let sql = sqlArr.join(" AND ");
    if (orSqlArr.length > 0) {
        sql += " OR (" + orSqlArr.join(" AND ") + ")";
        args = args.concat(orArgs);
    }
    sql = sql.replace(/ AND\s+OR /g, " OR ").replace(/^\s+OR /g, "");
    return { sql, args, ok: (sqlArr.length > 0 || orSqlArr.length > 0) };
};
exports.genWhere = genWhere;
const genData = (data) => {
    const newData = _.cloneDeep(data);
    for (const key in newData) {
        const value = newData[key];
        if (value != null && typeof value === "object") {
            newData[key] = JSON.stringify(value);
        }
    }
    return newData;
};
exports.genData = genData;
//# sourceMappingURL=helper.js.map