"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.del = exports.update = exports.insert = exports.select = exports.OR = void 0;
const types_1 = require("./types");
const helper_1 = require("./helper");
exports.OR = types_1.OR;
function select(table, fields, where, orderBy, limit) {
    let sql = `SELECT ${helper_1.getFields(fields)} FROM ??`;
    let args = [table];
    const whereSql = helper_1.genWhere(where);
    if (whereSql.ok !== false) {
        sql += ` WHERE ${whereSql.sql}`;
        args = args.concat(whereSql.args);
    }
    if (orderBy != null) {
        sql += " ORDER BY " + orderBy;
    }
    if (limit != null) {
        if (typeof limit === "string") {
            sql += " LIMIT " + limit;
        }
        else {
            const { offset, size } = limit;
            sql += ` LIMIT ${offset != null ? `${offset}, ` : ""}${size}`;
        }
    }
    return { sql, args };
}
exports.select = select;
function insert(table, data) {
    const sql = "INSERT INTO ?? SET ?";
    const genSetData = helper_1.genData(data);
    if (Object.keys(genSetData).length === 0) {
        throw new Error(helper_1.ErrorSetNull);
    }
    const args = [table, genSetData];
    return { sql, args };
}
exports.insert = insert;
function update(table, where, setData) {
    let sql = "UPDATE ?? SET ? WHERE";
    const genSetData = helper_1.genData(setData);
    if (Object.keys(genSetData).length === 0) {
        throw new Error(helper_1.ErrorSetNull);
    }
    let args = [table, genSetData];
    const whereSql = helper_1.genWhere(where);
    if (whereSql.ok !== false) {
        sql += ` ${whereSql.sql}`;
        args = args.concat(whereSql.args);
    }
    else {
        throw new Error(helper_1.ErrorWhereNull);
    }
    return { sql, args };
}
exports.update = update;
function del(table, where) {
    let sql = "DELETE FROM ?? WHERE";
    let args = [table];
    const whereSql = helper_1.genWhere(where);
    if (whereSql.ok !== false) {
        sql += ` ${whereSql.sql}`;
        args = args.concat(whereSql.args);
    }
    else {
        throw new Error(helper_1.ErrorWhereNull);
    }
    return { sql, args };
}
exports.del = del;
exports.default = {
    select, insert, update, del,
    OR: exports.OR,
};
//# sourceMappingURL=index.js.map